package model

import pasturePb "gitee.com/xuyiping_admin/go_proto/proto/go/backend/cow"

type EventMating struct {
	Id                int64                           `json:"id"`
	CowId             int64                           `json:"cowId"`
	DayAge            int64                           `json:"dayAge"`
	Lact              int8                            `json:"lact"`
	CalvingAge        int32                           `json:"calvingAge"`
	PlanDay           string                          `json:"planDay"`
	EndDay            string                          `json:"endDay"`
	RealityDay        string                          `json:"realityDay"`
	Status            pasturePb.IsShow_Kind           `json:"status"`
	MatingNumber      int64                           `json:"matingNumber"`
	MatingResult      pasturePb.MatingResult_Kind     `json:"matingResult"`
	ExposeEstrusType  pasturePb.ExposeEstrusType_Kind `json:"exposeEstrusType"`
	FrozenSemenNumber string                          `json:"frozenSemenNumber"`
	FrozenSemenCount  int32                           `json:"frozenSemenCount"`
	OperationId       int64                           `json:"operationId"`
	OperationName     string                          `json:"operationName"`
	Remarks           string                          `json:"remarks"`
	CreatedAt         int64                           `json:"createdAt"`
	UpdatedAt         int64                           `json:"updatedAt"`
}

func (e *EventMating) TableName() string {
	return "event_mating"
}

func NewEventMating(cow *Cow, planDay string) *EventMating {
	return &EventMating{
		CowId:            cow.Id,
		Lact:             int8(cow.Lact),
		PlanDay:          planDay,
		EndDay:           planDay,
		MatingResult:     pasturePb.MatingResult_Invalid,
		ExposeEstrusType: pasturePb.ExposeEstrusType_Same_Time,
		Status:           pasturePb.IsShow_No,
	}
}

func NewEventMatingList(cowList []*Cow, planDay string) []*EventMating {
	var matingList []*EventMating
	for _, cow := range cowList {
		matingList = append(matingList, NewEventMating(cow, planDay))
	}
	return matingList
}

type EventMatingSlice []*EventMating

func (e EventMatingSlice) ToPB(exposeEstrusTypeMap map[pasturePb.ExposeEstrusType_Kind]string) []*pasturePb.SearchMatingList {
	res := make([]*pasturePb.SearchMatingList, len(e))
	for i, v := range e {
		res[i] = &pasturePb.SearchMatingList{
			Id:                   int32(v.Id),
			CowId:                int32(v.CowId),
			DayAge:               int32(v.DayAge),
			Lact:                 int32(v.Lact),
			CalvingAge:           v.CalvingAge,
			PlanDay:              v.PlanDay,
			RealityDay:           v.RealityDay,
			ExposeEstrusType:     v.ExposeEstrusType,
			ExposeEstrusTypeName: exposeEstrusTypeMap[v.ExposeEstrusType],
			FrozenSemenNumber:    v.FrozenSemenNumber,
			Remarks:              v.Remarks,
			OperationId:          int32(v.OperationId),
			OperationName:        v.OperationName,
			CreatedAt:            int32(v.CreatedAt),
			UpdatedAt:            int32(v.UpdatedAt),
		}
	}
	return res
}

type CowMatingHeader struct {
	Id              string `json:"id"`
	CowId           string `json:"cowId"`
	PlanDay         string `json:"planDay"`
	Lact            string `json:"lact"`
	Status          string `json:"status"`
	BreedStatusName string `json:"breedStatusName"`
	BreedStatus     string `json:"breedStatus"`
	CowTypeName     string `json:"cowTypeName"`
	PenName         string `json:"penName"`
	DayAge          string `json:"dayAge"`
	CalvingAge      string `json:"calvingAge"`
	AbortionAge     string `json:"abortionAge"`
}

type CowMatingBody struct {
	Id              int64                      `json:"id"`
	CowId           int64                      `json:"cowId"`
	BreedStatus     pasturePb.BreedStatus_Kind `json:"breedStatus"`
	BreedStatusName string                     `json:"breedStatusName"`
	CowType         pasturePb.CowType_Kind     `json:"cowType"`
	CowTypeName     string                     `json:"cowTypeName"`
	PenId           int32                      `json:"penId"`
	PenName         string                     `json:"penName"`
	Lact            int32                      `json:"lact"`
	CalvingAge      int32                      `json:"calvingAge"`
	AbortionAge     int32                      `json:"abortionAge"`
	DayAge          int32                      `json:"dayAge"`
	Status          pasturePb.IsShow_Kind      `json:"status"`
}

type CowMatingBodySlice []*CowMatingBody

func (s CowMatingBodySlice) ToPB(
	cowTypeMap map[pasturePb.CowType_Kind]string,
	breedStatusMap map[pasturePb.BreedStatus_Kind]string,
	penMap map[int32]*Pen,
) []*CowMatingBody {
	res := make([]*CowMatingBody, len(s))
	for i, v := range s {
		res[i] = &CowMatingBody{
			Id:              v.Id,
			CowId:           v.CowId,
			CowType:         v.CowType,
			CowTypeName:     cowTypeMap[v.CowType],
			BreedStatus:     v.BreedStatus,
			BreedStatusName: breedStatusMap[v.BreedStatus],
			PenName:         penMap[v.PenId].Name,
			PenId:           v.PenId,
			Lact:            v.Lact,
			CalvingAge:      v.CalvingAge,
			AbortionAge:     v.AbortionAge,
			DayAge:          v.DayAge,
			Status:          v.Status,
		}
	}
	return res
}
